"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const config_manager_service_1 = require("./config-manager.service");
const Joi = require("@hapi/joi");
const constants_1 = require("./constants");
const defaultProcEnv = {
    NODE_ENV: 'test',
};
const config1 = {
    TEST1: {
        validate: Joi.string(),
        required: true,
    },
    TEST2: {
        validate: Joi.number(),
        required: true,
    },
    TEST3: {
        validate: Joi.number(),
        required: false,
        default: 3333,
    },
    TEST4: {
        validate: Joi.string(),
        required: false,
        default: '4444',
    },
};
const config2 = {
    TEST1: {
        validate: Joi.string(),
        required: true,
    },
    TEST2: {
        validate: Joi.number(),
        required: true,
    },
    TEST3: {
        validate: Joi.number(),
        default: 3333,
    },
    TEST4: {
        validate: Joi.string(),
        default: '4444',
    },
};
const setProperty = (object, property, value) => {
    const originalProperty = Object.getOwnPropertyDescriptor(object, property);
    Object.defineProperty(object, property, { value });
    return originalProperty;
};
const mockExit = jest.fn();
setProperty(process, 'exit', mockExit);
describe('ConfigService', () => {
    describe('It compiles and starts', () => {
        test('should be defined', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            expect(configService).toBeDefined();
        }));
    });
    describe('Test Module Configuration', () => {
        test('Should throw if no envKey or NODE_ENV in environment and not useFile', () => __awaiter(this, void 0, void 0, function* () {
            process.env = {};
            const configOpts = {
                useEnv: {
                    folder: 'testconfigs',
                },
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            try {
                const configService = new CfgService(configOpts);
            }
            catch (e) {
                expect(e.message).toEqual('Fatal error. No envKey specified, and `NODE_ENV` is not defined.');
            }
        }));
        test('Should not throw if no envKey or NODE_ENV in environment and useFile', () => __awaiter(this, void 0, void 0, function* () {
            process.env = {};
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            expect(configService).toBeDefined();
        }));
    });
    describe('Test validations', () => {
        test('Should throw on missing required env var', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test2.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            try {
                const configService = new CfgService(configOpts);
            }
            catch (e) {
                expect(e.missingKeys).toEqual(expect.arrayContaining(['"TEST1" is required, but missing']));
            }
        }));
        test('Should throw with string supplied on env var validating for number', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test3.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            try {
                const configService = new CfgService(configOpts);
            }
            catch (e) {
                expect(e.validationErrors).toEqual(expect.arrayContaining(['"TEST2" must be a number']));
            }
        }));
        test('Should throw with multiple missing required env vars', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test4.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            try {
                const configService = new CfgService(configOpts);
            }
            catch (e) {
                expect(e.missingKeys).toEqual(expect.arrayContaining([
                    '"TEST1" is required, but missing',
                    '"TEST2" is required, but missing',
                ]));
            }
        }));
        test('Should throw with missing required AND validation failure', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test5.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            try {
                const configService = new CfgService(configOpts);
            }
            catch (e) {
                expect(e.missingKeys).toEqual(expect.arrayContaining(['"TEST1" is required, but missing']));
                expect(e.validationErrors).toEqual(expect.arrayContaining(['"TEST2" must be a number']));
            }
        }));
    });
    describe('Test cascading', () => {
        test('Should pickup value from env when missing in dotenv', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NODE_ENV: 'test', TEST4: 'FOURFOURFOUR' };
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'throw',
                allowExtras: true,
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST4 = configService.get('TEST4');
            expect(TEST4).toEqual('FOURFOURFOUR');
        }));
        test('Should override dotenv value from env', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NODE_ENV: 'test', TEST1: 'def' };
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('def');
        }));
        test('Should show var in both environment and .env', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NODE_ENV: 'test', TEST1: 'def' };
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const trace = configService.trace();
            expect(trace.TEST1).toEqual({
                default: '--',
                dotenv: 'abc',
                env: 'def',
                isExtra: false,
                resolvedFrom: 'env',
                resolvedValue: 'def',
            });
        }));
        test('Should pick up default when missing', () => __awaiter(this, void 0, void 0, function* () {
            process.env = defaultProcEnv;
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const trace = configService.trace();
            expect(trace.TEST3).toEqual({
                default: config1.TEST3.default,
                dotenv: '--',
                env: '--',
                isExtra: false,
                resolvedFrom: 'default',
                resolvedValue: config1.TEST3.default,
            });
        }));
        test('Should override default when present in environment', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NODE_ENV: 'test', TEST4: 'FOUR' };
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const trace = configService.trace();
            expect(trace.TEST4).toEqual({
                default: config1.TEST4.default,
                dotenv: '--',
                env: 'FOUR',
                isExtra: false,
                resolvedFrom: 'env',
                resolvedValue: 'FOUR',
            });
        }));
        test('Should override default when present in dotenv', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test6.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const trace = configService.trace();
            expect(trace.TEST4).toEqual({
                default: config1.TEST4.default,
                dotenv: 'FOURTYFOUR',
                env: '--',
                isExtra: false,
                resolvedFrom: 'dotenv',
                resolvedValue: 'FOURTYFOUR',
            });
        }));
        test('Should pick up default when missing (with required omitted)', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config2;
                }
            }
            const configService = new CfgService(configOpts);
            const trace = configService.trace();
            expect(trace.TEST3).toEqual({
                default: config2.TEST3.default,
                dotenv: '--',
                env: '--',
                isExtra: false,
                resolvedFrom: 'default',
                resolvedValue: config1.TEST3.default,
            });
        }));
        test('Should override default when present in environment (with required omitted)', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NODE_ENV: 'test', TEST4: 'FOUR' };
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config2;
                }
            }
            const configService = new CfgService(configOpts);
            const trace = configService.trace();
            expect(trace.TEST4).toEqual({
                default: config2.TEST4.default,
                dotenv: '--',
                env: 'FOUR',
                isExtra: false,
                resolvedFrom: 'env',
                resolvedValue: 'FOUR',
            });
        }));
        test('Should override default when present in dotenv (with required omitted)', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test6.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config2;
                }
            }
            const configService = new CfgService(configOpts);
            const trace = configService.trace();
            expect(trace.TEST4).toEqual({
                default: config2.TEST4.default,
                dotenv: 'FOURTYFOUR',
                env: '--',
                isExtra: false,
                resolvedFrom: 'dotenv',
                resolvedValue: 'FOURTYFOUR',
            });
        }));
    });
    describe('Test extras', () => {
        test('Should allow extra vars when allowExtras is true', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test7.env',
                onError: 'throw',
                allowExtras: true,
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const trace = configService.trace();
            expect(trace.EXTRA).toEqual({
                default: '--',
                dotenv: 'abc',
                env: '--',
                isExtra: true,
                resolvedFrom: 'dotenv',
                resolvedValue: 'abc',
            });
        }));
        test('Should throw when extra vars present and allowExtras is default', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test7.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            try {
                const configService = new CfgService(configOpts);
            }
            catch (e) {
                expect(e.message).toEqual('Invalid Configuration');
            }
        }));
        test('Should throw when extra vars present and allowExtras is false', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test7.env',
                onError: 'throw',
                allowExtras: false,
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            try {
                const configService = new CfgService(configOpts);
            }
            catch (e) {
                expect(e.message).toEqual('Invalid Configuration');
            }
        }));
    });
    describe('Test onError options', () => {
        test('Should report errors and continue', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'continue',
                allowExtras: false,
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('abc');
        }));
        test('Should exit on error when onError = "throw"', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test7.env',
                onError: 'exit',
                allowExtras: false,
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            expect(process.exit).toBeCalled();
        }));
        test('Should exit on error when onError is omitted', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test7.env',
                allowExtras: false,
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            expect(process.exit).toBeCalled();
        }));
        test('Should exit on error when onError is invalid', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/test7.env',
                onError: 'junk',
                allowExtras: false,
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            expect(process.exit).toBeCalled();
        }));
    });
    describe('Test useEnv', () => {
        test('Should work with useEnv and specifying NODE_ENV envKey and no folder', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NODE_ENV: 'test1' };
            const configOpts = {
                envKey: 'NODE_ENV',
                useEnv: true,
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('abc');
        }));
        test('Should work with useEnv and specifying NODE_ENV envKey', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NODE_ENV: 'test1' };
            const configOpts = {
                envKey: 'NODE_ENV',
                useEnv: true,
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('abc');
        }));
        test('Should work with useEnv and specifying a folder', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NODE_ENV: 'test1' };
            const configOpts = {
                envKey: 'NODE_ENV',
                useEnv: {
                    folder: 'config',
                },
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('abc');
        }));
        test('Should work with useEnv and specifying non NODE_ENV envKey', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NONNODE_ENV: 'test1' };
            const configOpts = {
                envKey: 'NONNODE_ENV',
                useEnv: true,
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('abc');
        }));
        test('Should work with useEnv and defaulting to NODE_ENV ', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NODE_ENV: 'test1' };
            const configOpts = {
                useEnv: true,
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('abc');
        }));
        test('Should work with useEnv, no NODE_ENV provided, and default environment ', () => __awaiter(this, void 0, void 0, function* () {
            process.env = {};
            const configOpts = {
                useEnv: true,
                onError: 'throw',
                defaultEnvironment: 'test1',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('abc');
        }));
        test('Should work with useEnv and ignore default environment if NODE_ENV defined', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NODE_ENV: 'test1' };
            const configOpts = {
                useEnv: true,
                onError: 'throw',
                defaultEnvironment: 'test2',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('abc');
        }));
    });
    describe('Test useFunction', () => {
        test('Should work with useFunction and default envKey (NODE_ENV)', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NODE_ENV: 'test1' };
            function configResolver(rootFolder, environment) {
                return `${rootFolder}/config/${environment}.env`;
            }
            const configOpts = {
                useFunction: configResolver,
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('abc');
        }));
        test('Should work with useFunction and non-default envKey', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { ENVIRONMENT: 'test1' };
            function configResolver(rootFolder, environment) {
                return `${rootFolder}/config/${environment}.env`;
            }
            const configOpts = {
                envKey: 'ENVIRONMENT',
                useFunction: configResolver,
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('abc');
        }));
    });
    describe('Test missing files', () => {
        test('Should throw when useFile points at bad file', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                useFile: 'config/nonsense.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            try {
                const configService = new CfgService(configOpts);
            }
            catch (e) {
                expect(e.message).toMatch(/Fatal error loading environment. The following file is missing:/);
            }
        }));
        test('Should throw when useEnv has a bad envKey', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                envKey: 'BAD_KEY',
                useEnv: {
                    folder: '',
                },
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            try {
                const configService = new CfgService(configOpts);
            }
            catch (e) {
                expect(e.message).toMatch(/Bad environment key:/);
            }
        }));
        test('Should throw when useEnv has a bad envKey and allowMissingEnvFile is true', () => __awaiter(this, void 0, void 0, function* () {
            process.env = Object.assign({}, defaultProcEnv);
            const configOpts = {
                envKey: 'BAD_KEY',
                useEnv: {
                    folder: 'srcx',
                },
                onError: 'throw',
                allowMissingEnvFile: true,
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            try {
                const configService = new CfgService(configOpts);
            }
            catch (e) {
                expect(e.message).toMatch(/Bad environment key:/);
            }
        }));
        test('Should throw when useEnv key points to bad file', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { USELESS_KEY: 'abc' };
            const configOpts = {
                envKey: 'USELESS_KEY',
                useEnv: {
                    folder: '',
                },
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            try {
                const configService = new CfgService(configOpts);
            }
            catch (e) {
                expect(e.message).toMatch(/Fatal error loading environment. The following file is missing:/);
            }
        }));
        test('Should not throw when useEnv key points to bad file and allowMissingEnvFile is true', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { NODE_ENV: 'nonsense', TEST1: 'eee', TEST2: '555' };
            const configOpts = {
                envKey: 'NODE_ENV',
                useEnv: {
                    folder: '/nowhere',
                },
                onError: 'throw',
                allowMissingEnvFile: true,
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('eee');
        }));
        test('Should throw when useFunction points to bad file', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { USELESS_KEY: 'abc' };
            function configResolver(rootFolder, environment) {
                return `${rootFolder}/badfolder/${environment}.env`;
            }
            const configOpts = {
                envKey: 'USELESS_KEY',
                useFunction: configResolver,
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            try {
                const configService = new CfgService(configOpts);
            }
            catch (e) {
                expect(e.message).toMatch(/Fatal error loading environment. The following file is missing:/);
            }
        }));
        test('Should not throw when useFunction points to bad file and allowMissingEnvFile is true', () => __awaiter(this, void 0, void 0, function* () {
            process.env = { USELESS_KEY: 'abc', TEST1: 'eee', TEST2: '555' };
            function configResolver(rootFolder, environment) {
                return `${rootFolder}/badfolder/${environment}.env`;
            }
            const configOpts = {
                envKey: 'USELESS_KEY',
                useFunction: configResolver,
                onError: 'throw',
                allowMissingEnvFile: true,
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            const TEST1 = configService.get('TEST1');
            expect(TEST1).toEqual('eee');
        }));
    });
    describe('Test trace() in different environments', () => {
        test('trace() should be called in non production', () => __awaiter(this, void 0, void 0, function* () {
            const mockDbg = jest.fn();
            setProperty(constants_1.dbg, 'trace', mockDbg);
            process.env = { NODE_ENV: 'something', DEBUG: 'trc' };
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            expect(mockDbg).toBeCalledTimes(1);
        }));
        test('trace() should not be called in production', () => __awaiter(this, void 0, void 0, function* () {
            const mockDbg = jest.fn();
            setProperty(constants_1.dbg, 'trace', mockDbg);
            process.env = { NODE_ENV: 'production', DEBUG: 'trc' };
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            expect(mockDbg).toBeCalledTimes(0);
        }));
        test('trace() should be called in production with TRACE_PRODUCTION', () => __awaiter(this, void 0, void 0, function* () {
            const mockDbg = jest.fn();
            setProperty(constants_1.dbg, 'trace', mockDbg);
            process.env = {
                NODE_ENV: 'production',
                DEBUG: 'trc',
                TRACE_PRODUCTION: 'true',
            };
            const configOpts = {
                useFile: 'config/test1.env',
                onError: 'throw',
            };
            class CfgService extends config_manager_service_1.ConfigManager {
                provideConfigSpec() {
                    return config1;
                }
            }
            const configService = new CfgService(configOpts);
            expect(mockDbg).toBeCalledTimes(1);
        }));
    });
});
